/*********************************************************************
    Fichero:    FoM V1  (FunSECA)
    Autor:      Fun-o-Matic 
    Fecha:      Nov2000 - Julio2001
    Proposito:  Emular a una SECA SmartCard para fines didcticos.
    Compilador: AVR-GCC + Make2 de gcctest.zip
    Hardware:   FunCard y Apollo de funcard.net
**********************************************************************/

#include <io.h>
#include "types.h"
#include "funseca.h"
#include "funio.h"
#include "decrip.h"
#include "ee_ext.h"
#include "ee_int.h"

#define AUTOPPUA
#define MODO_LOG
#define AUTOPBM

#define MAX_COMMAND_LENGHT 0x5A  // (+5 de la cabecera = 0x5f)

u08 funomatic[16-2]={'F','u','n','-','o','-','m','a','t','i','C',' ','v','1'};

// Simular registro de compra
u08 ppv_record[14]={0xB1,0x00,0xFF,0xFF,0xFF,0xFE,0x05,0x00,0x00,0xFF,0xFF,0x00,0x00,0x04};

#define EVENT_ID1              2
#define EVENT_ID2              3
#define SESSION_ID             4
#define SESSION_DATE1          9
#define SESSION_DATE2         10

// Pin  7799
#define PIN_INFO_H   0x1E
#define PIN_INFO_L   0x77

// Punteros a las funciones de comunicacin
void (*ptrSend)(u08);
u08  (*ptrReceive) (void);

// keyleds:
// - 0: always off (low power supply decos) (PIN:9999)
// - 1: show operation key in use (PIN:9998)
// - 2: show current provider index (PIN:9997)
u08 keyleds=1;

// info mode (On Screen Display):
// - 0: show provider + expiration date (mode info off)
// - 1: show Channel id, operation Key, etc (mode info on)
u08 infoMode=0x00;

// currentProvider, currentKey & currentChannel for info mode
u08 currentProvider=0x00;
u08 currentKey=0x00;
u08 currentChannel=0x00; 

// stores the last processed instruction
u08 lastInst=0x00; 

// Permite utilizar el phoenix para realizar operaciones de I/O enla eeprom externa.
u08 support_phoenix=0x01;

// Auto-Actualizar el PBM.
// Si el Channel Boundle ID permanece cte en valor y posicin
// durante tres ECM seguidas se autoactualiza 
// Solo el que este en primera posicin.
#ifdef AUTOPBM
u08 autoPBM = 0;
u08 repeat_bid = 0;
u08 last_bid= 0xff;
#endif

// PAra el cloqueo de taquillas
u08 decodeX    = FALSE;
u08 needPin    = TRUE;
u08 pinChecked = FALSE;

// Answer to Reset
// Indica que la tarjeta es una versin 4.0
u08 ATR[16]={0x3B,0xF7,0x11,0x00,0x01,0x40,0x96,0x54,0x30,0x04,0x0E,0x6C,0xB6,0xD6, 0x90, 0x00};

// Datos para la Inst 0x0A: Sistema Operativo de la tarjeta
#define TOTAL_VERSION 68 
u08 version[TOTAL_VERSION]= {
  0x61,0x19,0xad,0xca,0x15,0x2f,0x10,0x03,
  0x00,0x00,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,'(' ,'c' ,')' ,'F' ,'u' ,'n' ,
  '-' ,'o' ,'-' ,'m' ,'a' ,'t' ,'i' ,'C' ,
  ' ' ,'2' ,'0' ,'0' ,'1' ,' ' ,'V' ,'1' ,
  '.' ,'0' ,'0' ,0x00 };
  
// Buffer de entrada y salida
u08 buffer[MAX_COMMAND_LENGHT];

// clave en uso (PK + SK)
u08 Key[8+8];

// hash-buffers para la comprobacin de la firma
u08 hash[8];
u08 initial_hash[8];

// ControlWords desencriptadas para la 0x3A
u08 CW[8+8];

// comando enviado por la 0x34
u08 Comando34[3];

// status bytes
u08 sw1=0x90,sw2=0x00;

void sendString(u08 *str, u08 len)
{
  do { (*ptrSend)(*str++); } while (--len);
}

void receiveString(u08 len)
{
  u08 *dst=buffer;

  do { *dst++ = (*ptrReceive)(); } while (--len);
}

void fillBuffer(u08 begin_idx ,u08 end_idx, u08 fillByte)
{
  u08 cnt;

  for (cnt=begin_idx; cnt<end_idx; cnt++)
    buffer[cnt]=fillByte;
}

void _memcpy(u08 *dst,u08 *src,u08 n)
{
 while(n){
   *dst=*src;
   dst++;
   src++;
   n--;
  } 
}


u16 getProvAddr(u08 p1)
{
  u16 res;

  p1 &= 0x0f;
  for (res = EE_ENTETY_0; p1; p1--)
    res += EE_ENTETY_SIZE;

  return res;
}


u16  getPKAddr(u08 p1, u08 p2)
{
  u16 key_offset = getProvAddr(p1) + EE_ENT_KEY_0;

  //key_index*=8, at908515 opt, se pierde el bit 7 (superencriptacin)
  p2 <<= 1; p2 <<= 1; p2 <<= 1;

  return (key_offset + p2);
}


void  decode(u08 p2, u08 len)
{
  u08 *src=buffer;

  if (p2&0x80){
    while (len>8){
      decrip(Key,src);
      len-=8;
      src+=8;
    }
  }
}

void getKey(u08 p1, u08 p2)
{
  u16 key_offset=getPKAddr(p1,p2);

  ee_readString(Key,key_offset,8);
  ee_readString(Key+8,(p1&0x10)?key_offset+(16*8):key_offset,8);
}


u08 xor_hash(u08 len, u08 r)
{
  u08 cnt;

  decrip(Key,hash);
  len-=r;
  for (cnt=0; cnt<r; cnt++)
    hash[cnt]^=buffer[len+cnt];

  return len;
}

u08 signatureCheck(u08 p1, u08 len)
{
  u08 cnt;
  u08 resto=len&7;
  u16 addr;


  len-=8; // indice al nano 0x82
  if (buffer[len-1]!=0x82)
    return 0;
  
  _memcpy(hash,buffer+len,8);
  
  // calculo inverso de la firma

  if (resto){
    len=xor_hash(len,resto);
  }

  while (len>0){
    len=xor_hash(len,8);
  }

  // Calcular el hashbuffer inicial
  for (cnt=0; cnt<8; cnt++){
    initial_hash[cnt]=0;
  }

  addr=getProvAddr(p1) + EE_ENT_PPUA;
  p1&=0xE0;
  if (p1==0x20){
    // UA + 00 00
    ee_readString(initial_hash, EE_NUM_TARJETA,6);
  } else
  if (p1==0x40){
    // PPUA + 00 00 00 00
    ee_readString(initial_hash,addr,4);
  } else
  if (p1==0x60){
    // SA + 00 00 00 00 00
    ee_readString(initial_hash,addr,3);
  }

  // Si firma correcta -> hash buffer calculado = hash buffer inicial
  for (cnt=0; cnt<8; cnt++){
    if (initial_hash[cnt]!=hash[cnt])
      return 0;
  }
  
  return 1;
}



void updateKey(u08 idx, u08 p1, u08 sk)
{
  u16 addr;

  // desencriptar la clave
  _memcpy(CW,&buffer[idx+2],8);
  decrip(Key,CW);

  // calcular direccin
  addr=getPKAddr(p1,(buffer[idx+1]&0x0F));
  addr=sk?(addr+(16*8)):addr;

  // actualizar si es diferente a la almacenada
  ee_updateString(addr,CW,8);
}


u08 keyExist(u08 p1, u08 p2)
{
  // 0x00 si la clave no existe
  // 0x50 si slo PK
  // 0xf0 si tambien SK (nota: la PK puede ser 8*0x00)

  u08 cnt,res=0x00;

  // Cargar Pk&Sk
  getKey(p1|0x10,p2);

  for (cnt=0; cnt<16; cnt++){
    if (Key[cnt]){
      if (cnt<8)
        res=0x50;
      else res=0xf0;
   }
 }
  return res;
}


u08 nibble2ascii(u08 nibble)
{
  return (nibble<0x0a)?(nibble+'0'):(nibble+('A'-0x0a));
}

void byte2ascii(u08 value, u08 idx)
{
  u08 *dst=&buffer[idx];

  *dst++=nibble2ascii(value>>4);
  *dst=nibble2ascii(value&0x0f);
}

void setupInfo(u08 p1, u08 p3)
{
  u16 addr;
  u08 i,j,n;

  if (infoMode){
    // Rellenar con espacios campo nombre
    fillBuffer(2,(2+16)-1,0x20);
    addr=getProvAddr(currentProvider);
    // PPUA + Fecha
    ee_readString(buffer+(2+16),addr+EE_ENT_PPUA,4+2);

    p1&=0x0f;
    if (p1==0x01){
      _memcpy(buffer+(2+1),funomatic,16-2);
    } else
    if (p1==0x02){
      ee_readString(buffer + 2, addr + EE_ENT_NAME, 16);
    } else
    if (p1==0x03){
      if (currentChannel){
        // "P:xxxx/X  K:xx  "
        ee_readString(buffer+2,0x002f,16);
        byte2ascii(currentChannel,2+7);
       } else {
        // "P:xxxx    K:xx  "
        ee_readString(buffer+2,0x003f,16);
       }

       // Rellenamos Ident del proveedor
       byte2ascii(ee_read(addr),2+2);
       byte2ascii(ee_read(addr+1),2+4);

       // Rellenamos indice de la clave
       byte2ascii(currentKey,2+12);
    } else
    if (p1<0x08){
      if (keyExist(currentProvider,currentKey)){
        // indice inicial
        i=(p1-4)<<2;
        // indice final
        n=i+4;
        // convertir a ascii
        for (j=2; i<n; i++, j+=3){
          byte2ascii(Key[i],j);
        }
      }
    } else {
      // Aadimos informacin del ltimo evento
      byte2ascii(ppv_record[EVENT_ID1],2);
      byte2ascii(ppv_record[EVENT_ID2],4);
      byte2ascii(ppv_record[SESSION_ID],6);
    }
  } // mode info ends

  addr=getProvAddr(p1);

  // Ident
  ee_readString(buffer,addr,2);

  if (!infoMode){
    // Provider name en buffer[2..18]
    ee_readString(buffer + 2, addr + EE_ENT_NAME, 16);
    // PPUA + Fecha
    ee_readString(buffer+(2+16),addr + EE_ENT_PPUA,4+2);
  }
}

u08 cardHasExpired(u08 p1, u08 nano_idx)
{
  u08 expdate_h,expdate_l;
  u08 actdate_h,actdate_l;
  u08 *ptr;
  u16 addr;

  // get card expiration date
  addr=getProvAddr(p1) + EE_ENT_SUBS_DATE;
  expdate_h=ee_read(addr);
  expdate_l=ee_read(addr+1);

  // get ECM or EMM command date
  ptr=&buffer[nano_idx];
  actdate_h=*(ptr+1);
  actdate_l=*(ptr+2);

  return (actdate_h>expdate_h || (actdate_h==expdate_h && actdate_l>expdate_l));
}

u08 isCustWPActive(u08 nanof0_idx, u08 custwp)
{
  return rightShift(buffer[(nanof0_idx + 32) - rightShift(custwp,3)],custwp&7)&1;
}

u08 processNanos(u08 p1,u08 ECMorEMM)
{
 u08 nano;
 u08 idx=0;
 u08 rightsAdquired = 0;
 u08 nanos_13_31 = 1;
 u08 pnanos = 0; // processed nanos
 u16 addr;
 u08 cnt_n13 = 0; // nanos 0x13 procesados, para autopbm
 u08 nano_2c = 1; // controla el bloqueo de las X
 
 while (1){

  nano = buffer[idx];
   
  if (nano==0x82){
    pnanos++;   
    return pnanos;
  } else
 
  // ECM nanos - - - - - - - - - - - - - - - - - - - - - - - - - - -   
   
  if (ECMorEMM){
  
  if (nano==0x2c){
    // x channel
    nano_2c =  decodeX;
  } else
  
  if (nano==0x04 && !rightsAdquired){
    // free the channel
    rightsAdquired = 1;
    pnanos++;   
  } else

  if (nano==0x19 && !rightsAdquired){
    // preview mode
    rightsAdquired = 1;
    pnanos++;   
  } else
    
  if (nano==0x27 && !rightsAdquired){
    // test card expiration date
    pnanos++;   
    if (cardHasExpired(p1,idx)){
      sw1=0x93; sw2=0x01; 
      return pnanos;
    } 
    
    ppv_record[SESSION_DATE1] = buffer[idx+1];
    ppv_record[SESSION_DATE2] = buffer[idx+2];
  } else
   
  if (nano==0x13 && !rightsAdquired && nanos_13_31){
    // test channel boundle identifier
    u08 pbm,bid;
    if ((bid=buffer[idx+1])<64){
      addr=getProvAddr(p1)+ (EE_ENT_PBM + 7) - rightShift(bid,3);
      pbm=ee_read(addr);
      if (rightShift(pbm,bid&7)&1){
        RED_INFO_LED_OFF;
        rightsAdquired=1;
        currentChannel=bid;
      } else {
        #ifdef AUTOPBM
        if(ee_read(EE_FLAGS) & FLG_AUTOPBM){
          // Channel boundle no activo
          // Algoritmo de autoactualizacion si es el primer 0x13
          RED_INFO_LED_ON;
        
          if (cnt_n13==0){
            // almacenar la cuenta de apariciones del mismo bid seguido
            repeat_bid = (last_bid==bid)? (repeat_bid+1) : 0;
            // si el bid no cambia en tres ecm seguidas->autoPBM
            autoPBM = (repeat_bid==2); 
            last_bid = bid;
           } 
         
           if (autoPBM){
             u08 tmp;
             tmp=pbm|rightShift(0x80,7-(bid&7));
             if(tmp!=pbm)
               ee_write(addr,tmp);
             autoPBM=0x00;
             repeat_bid=0x00;
           }
        }
        #endif
      } // end of else
    }
    
    pnanos++;  
    cnt_n13++;
  } else
   
  if (nano==0x31 && !rightsAdquired && nanos_13_31){
    ppv_record[SESSION_ID] = buffer[idx+3];
    // Ya no es necesario verificar el 0x13, 0x31
    nanos_13_31 = 0;
    pnanos++;   
  } else
 
  if (nano==0xD1){
    pnanos++; 
   
    if ((rightsAdquired || !nanos_13_31) && (nano_2c)){
      u08 *ptr;  
      ptr=&buffer[idx+1];
      decrip(Key,ptr);
      decrip(Key,ptr+8);
      _memcpy(CW,ptr,16);
    } else{
      // rights no adquired
      sw1=0x93; sw2=0x02; 
      return pnanos;
    }
  } 
  
  } // end of ECM nanos
  

  // EMM nanos - - - - - - - - - - - - - - - - - - - - - - - - - - -
  
  else {
    
  if (nano==0x21){
    // update expiration date
    addr=getProvAddr(p1) + EE_ENT_SUBS_DATE;
    ee_updateString(addr,&buffer[idx+1],2);
    pnanos++;
  } else
 
  if (nano==0x22){
    // test card expiration date
    pnanos++;   
    if (cardHasExpired(p1,idx)){
        sw1=0x93; sw2=0x01; 
        return pnanos;
    }
  } else

  if (nano==0x41){
    // update PPUA
    addr=getProvAddr(p1)+ EE_ENT_PPUA;
    
    if (ee_updateString(addr,&buffer[idx+1],4)){
       sw2 = 0x19; // 90 19
    }
    pnanos++;
  } else
  
  if (nano==0x90 || nano==0x91){
    // update primary or secondary key 
    pnanos++;
    updateKey(idx,p1,nano&0x01);
  } else 
  
  if (nano==0xF0){
    // CustWP-Bitmap
    u08 custwp;

    pnanos++;
    addr=getProvAddr(p1)+(EE_ENT_PPUA + 3);
    custwp = ee_read(addr);
    
    if (!isCustWPActive(idx, custwp)){
      // Si no esta activo y es distinto a cero -> 90 09

      #ifdef AUTOPPUA
      // PPUA Autoupdate: autoactualiza el custwp ( + 46 bytes)
      u08 cnt;
      
      cnt = ee_read(EE_FLAGS);
      if(cnt & FLG_AUTOPPUA){
        cnt = custwp;
        do{
           cnt++;
           if (isCustWPActive(idx,cnt)){
             // Encontrado un CustWP Activo! Actualizando PPUA!
             ee_write(addr,cnt);
             // enciende led rojo, se apagara en la proxima 0x3c
             RED_INFO_LED_ON; 
             cnt = custwp; // forzar la salida
           }
        } while (cnt!=custwp);
      }
      
      #endif
      
      sw2 = 0x09;
      return pnanos;
    }
  } // end of nano f0
  
  } // end of EMM nanos
    
 

  // next nano
  nano>>=4;
  if (nano>0xC) nano=(nano-0x0b)<<3;
  idx+=(nano+1);
 }
}

#ifdef MODO_LOG
#include "log.h"
#endif

int main(void)
{
  u08 inst=0,p1=0,p2=0,p3=0;
  u08 numProviders;  
  u08 tmp;
  u08 cla=0;

  // Desactiva el comparador analogico :P
  sbi(ACSR,7);

  // Configura Puertos B    y D

  // El puerto B    :
  //    - leds de colores (pins 2,3,4)
  //    - i/o (pin 6)

  outp(0x1C,DDRB);    // 0b00011100   
  outp(0x1C,PORTB);   // 0b01011100

  // El puerto D    :
  //    - KeyLeds (pins 0,1,2,3)
  outp(0x0F,DDRD);   // 0b00001111
  outp(0x0F,PORTD);  // 0b00001111

  // init eeprom externa

  /*-----------------------------------------------------------------------------
  | Deteccin modo Apollo:
  | 
  | Se comprueba el estado del bit 7 y bit 5 del puerto B. Si el 7 est a 0 y
  | el 5 a 1, alguien los ha cambiado (FunStudio) -> modo Apollo
  | 
  +------------------------------------------------------------------------------------*/
  
  if(0x80 == (inp(PINB) & 0xA0))
  {   
    sbi(DDRB,6);
    sbi(PORTB,6);
    RED_INFO_LED_ON;
    ptrSend    = &SendA;
    ptrReceive = &ReceiveA;
    cbi(PORTB,6);
  }
  else
  {    
    // Modo Phoenix
    ptrSend    = &Send;
    ptrReceive = &Receive;

    // Enviamos ATR
    ORANGE_INFO_LED_ON;
    sendString(ATR,16);
    ORANGE_INFO_LED_OFF;
//    Send(0x90);
//    Send(0x00);
  }
  
  extee_init();
  
  numProviders = ee_read(EE_NUM_ENTETIES) - 1;
  
  // Para los decos sin PIN. Permite bloquear los canales X
  decodeX = (ee_read(EE_FLAGS) & FLG_BLOQUEAR_TX);
  
  #ifdef MODO_LOG
  //entraremos en modo log en la prxima instruccin si el primer byte != 0
  modo_log=ee_read(EE_FLAGS) & FLG_AUTOLOG;
  leer_config_log(numProviders);
  #endif
  
  while (1){
    
    // status word by default
    sw1=0x90;
    sw2=0x00;

    GREEN_INFO_LED_OFF;     // apaga 'busy' led    
    cla=(*ptrReceive)();    // C1 en SECA
    GREEN_INFO_LED_ON;      // enciende 'busy' led

    inst=(*ptrReceive)();   // Instruccion
    p1=(*ptrReceive)();     // parametro 1
    p2=(*ptrReceive)();     // parametro 2
    p3=(*ptrReceive)();     // longitud del comando

    if (cla!=0xC1){
      // Clase no soportada: 6E 00
      lastInst=0;
      sw1=0x6E;
      goto TX_Data;
    }
    
    if (inst&1){
      // El primer bit de la inst debe ser cero
      lastInst=0;
      sw1=0x6D;
      goto TX_Data;
    }
    
    if (!p3 || p3>MAX_COMMAND_LENGHT){
      // Longitud de entrada incorrecta: 67 00
      lastInst=0;
      sw1=0x67;
      goto TX_Data;
    }

    (*ptrSend)(inst);  // Enviar ACK
    
    if (inst&2){
      // CARD->CAM
      fillBuffer(0,MAX_COMMAND_LENGHT,0xFF);
    } else {
      // CAM->CARD
      receiveString(p3); 
      #ifdef MODO_LOG
      // salvamos el buffer para el log
      _memcpy(&log_buffer[5],buffer,p3);
      #endif
    }

    /*-----------------------------------------------------------------------------
    | En principio, el tratar esta posibilidad aqu no debe producir problemas
    | Con !support_phoenix nos aseguramos poder mandar todas las direcciones
    | para las INS 20 y 22
    +-----------------------------------------------------------------------------*/
    if (((p1 & 0x0F) > numProviders) && (!support_phoenix)){
    // proveedor no soportado 90 04
      sw2=0x04;
      goto TX_Data;
    }

    
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x34: Especificacin de la Peticin de Datos
    if (inst==0x34){
      // Slo soportamos dos peticiones:
      //   00 00 00 : Provider Package Bitmap Records
      //   04 00 01 : SECA Startup Records
      // La peticin se almacena en Comando34
      lastInst=0;
      
      if (p1&0x0F){
        // proveedor debe ser 0
        sw2=0x04;
        goto TX_Data;
      }

      if (p3!=3){
        // Longitud Incorrecta de la entrada: 67 00
        sw1=0x67;
        goto TX_Data;
      }
      
      _memcpy(Comando34,buffer,3);
      lastInst=inst;
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x32: Peticin de Fecha
    if (inst==0x32){
      
      if (p2){
        // valor de p2 incorrecto (p2!=0) 94 02
        lastInst=0;
        sw1=0x94;
        sw2=0x02;
        goto TX_Data;
      }

      
      if (lastInst!=0x34){
        // Instruccin previa incorrecta 90 14
        lastInst=0;
        sw2=0x14;
        goto TX_Data;
      }

      // Se puede volver a llamar a una ins 0x32 tras una ins 0x32 correctamente procesada.
      lastInst=0x34;
      
      // Comando 00 - - - - - - - - - - - - - - - - - - - - 
      if (Comando34[0]==0x00){
        // Comando 00
        // Datos:  00 00 -> Provider Package Bitmap Records
        
        if (p3<0x0A){
          // No cabe, mandar 0x03
          buffer[0]=0x03;
        } else {
          if (p1){
            // ok, todos menos SECA
            buffer[0]=0x83;
            ee_readString(buffer + 1, getProvAddr(p1) + EE_ENT_PBM, 8);
            buffer[0x0A-1]=0x04;
          } else {
            // a SECA nada ;)
            buffer[0]=0x04;
          }
        }
      } else
      // Comando 03 - - - - - - - - - - - - - - - - - - - - 
      if (Comando34[0]==0x03){
        // Comando 03
        // Datos:  xx xx -> Provider PPV Records (xx xx = EventID)
        if(p3<0x0e){
          buffer[0] = 0x03;
        } else {
          ppv_record[EVENT_ID1] = Comando34[1];
          ppv_record[EVENT_ID2] = Comando34[2];
          _memcpy(buffer,ppv_record,14);
          needPin = FALSE;
        }
      } else 
      // Comando 04 - - - - - - - - - - - - - - - - - - - - 
      if (Comando34[0]==0x04){
        // Commando 04
        // Datos:  00 00 -> SECA PPV Records
        //         00 01 -> SECA Startup Records
        //         00 02 -> SECA Records de Activacin
        if (Comando34[2]==0x01){
          // SECA Startup Records
          if (p3<0x0d){
            // No cabe, mandar 0x03
            buffer[0]=0x03;
          } else {
            if (p1){
              // Slo se admite p1=SECA
              buffer[0]=0x04;  
            } else {
              buffer[0]=0xb2;
              ee_readString(buffer + 1, EE_STARTUP_STR, 11);
              buffer[0x0D-1]=0x04;
            }
          } 
        }
      } // fin comando 04
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x0E: Nmero de serie de la tarjeta
    if (inst==0x0E){
      
      fillBuffer(0,2,0x00);
      // numero de serie esta en SECA.UA
      ee_readString(buffer + 2, EE_NUM_TARJETA, 6);
      support_phoenix=0x00;
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x16: Enumeracin de Proveedores
    if (inst==0x16){
      
      fillBuffer(0,5,0x00);
      
      tmp=numProviders + 1; //nmero de proveedores + seca
      buffer[2]=rightShift(0xff,16-tmp);
      buffer[3]=rightShift(0xff,(tmp<8?(8-tmp):0));
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x12: ProviderID (Ident)
    if (inst==0x12){

      setupInfo(p1,p3);
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x3C: ECM ControlWords encriptadas
    if (inst==0x3C){
      // p1 = provider number & pk-sk
      // p2 = key index & superencription

      needPin = TRUE;
      pinChecked = FALSE;
      // actualizar variables modo info
      currentProvider=p1&0x0F;
      currentKey=p2&0x0F;
      currentChannel=0;
      lastInst=inst;
      
           
      // comprueba existencia de claves
      tmp = keyExist(p1,currentKey);
      
      if (tmp==0x00){
        // No existe clave primaria 90 1D
        sw2=0x1D;
        goto TX_Data;
      } else
      if (tmp==0x50 && (p1&0x10)){
        // No existe clave secundaria 90 1F
        // excepcion: devuelve los 0x10 primeros bytes para 0x3a
        // Atencion: Ahora devuelve relleno. Cuidadin!
        sw2=0x1F;
        goto TX_Data;
      }

      getKey(p1,p2);
      decode(p2,p3);

      // info en KeyLeds
      keyleds=ee_read(EE_KEYLEDS);


      if (keyleds==0){
        outp(0x0F,PORTD);
      } else
      if (keyleds==1){
        outp(~(currentKey),PORTD);
      } else
      if (keyleds==2){
        outp(~(currentProvider),PORTD);
      }

      if (signatureCheck(p1,p3)){
        RED_INFO_LED_OFF;
        processNanos(p1,kECM);
      } else {
        // 90 02: Signature fails
        sw2=02; 
        RED_INFO_LED_ON;
      }
      
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x3A: ECM ControlWords desencriptadas
    if (inst==0x3A){

      // Verificar inst, anterior.
      if (lastInst!=0x3C){
        sw2=0x14;
        goto TX_Data;
      }

      if (p1!=0){
        sw2=0x04;
        goto TX_Data;
      }
      
      lastInst=inst;
      // enviar CW desencriptadas
      _memcpy(buffer,CW,16);
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x30: Transacciones protegidas por PIN
    if (inst==0x30){
      lastInst=inst;
      //  Introduccion/modificacion de PIN
      //   - El PIN se almacena en eeprom externa.
      //   - Utilizo los dos ultimos bytes de la UA
      //    del proveedor con indice 1 (sigue a SECA).
      if (buffer[14]==0x27){
        // Apagar Keyleds?
        if (buffer[15]==0x0f){
          ee_write(EE_KEYLEDS,0);
        }

        // Keyleds muestran indice de la clave en uso
        if (buffer[15]==0x0e){
          ee_write(EE_KEYLEDS,1);
        }

        // Keyleds muestran indice del proveedor actual
        if (buffer[15]==0x0d){
          ee_write(EE_KEYLEDS,2);
        }
      } else
      if ((buffer[14]==PIN_INFO_H) && (buffer[15]==PIN_INFO_L))
        infoMode=!(infoMode);
     
      #ifdef MODO_LOG  
      else tratar_PINS_log( buffer[14], buffer[15], numProviders );
      #endif
    
      /*-----------------------------------------------------------------------------
      | Bloqueo de taquillas oficial:
      |   Una vez que se bloquea una taquiila, el deco pedir PIN a la hora de la
      |   compra an cuando esa taquilla no est bloqueada. En una taquilla bloqueada,
      |   lo pide 2 veces: a la hora de entrar en la taquilla y a la hora de la compra.
      |
      | Lo que intentamos es simular el bloqueo de taquillas evitando la peticin del
      | PIN, a la hora de la compra, si esa taquilla no est bloqueada. En una bloqueada
      | lo seguir pidiendo 2 veces.
      |
      | A la hora de realizar la compra o volver a una taquilla ya comprada, el deco no 
      | nos da datos suficientes para poder saber en que situacin estamos. El sistema
      | que hemos adoptado garantiza, al menos, una peticin de PIN cuando se entre en
      | una taquilla bloqueada, ya sea para el proceso de compra o si cambiamos a una
      | taquilla bloqueada con un evento de compra ya activado. En contra partida, en alguna
      | situacin, puede darse el caso de que se pida PIN en el proceso de compra de una 
      | taquilla sin bloquear :-)
      |
      | Variables que controlan la situacin:
      |   needPin : Se activa al inicio, en cada INS 3C, si se introduce in PIN correcto
      |             o si en la comprobacin del PIN se encontraba sin activar needPIN o 
      |             pinChecked.
      |             Se desactiva con la INS 32, comando 4 (compra PPV)
      |
      |   pinChecked : Se activa si se introduce un PIN correcto o si en la comprobacin
      |                del PIN se encontraba sin activar needPIN o pinChecked.
      |                Se desactiva al inicio y en cada INS 3C.
      |
      | Se comprueba el PIN si cualquiera de las dos variables se encuentran 
      | activas en el momento de la peticin del mismo.
      +------------------------------------------------------------------------------------*/

      if ((pinChecked) || (needPin)){
        if (mem2eeprom_cmp(EE_PIN,&buffer[14],2)==0){
          // ultimo PIN validado = PIN almacenado   ?
          if (mem2eeprom_cmp(EE_PIN,&buffer[6],2)==1){
            // modificacin del PIN
            ee_writeString(EE_PIN,&buffer[14],2);
          } 
          else {
            sw2=0x10; // no validar PIN: 9010
          }
        } else {
        needPin = pinChecked = TRUE;
        }
      }
      else {                            // Aunque parezca mentira
        needPin = pinChecked = TRUE;    // esta lnea ocupa -2 bytes !!??
      } // fin comprobacin pin
      
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x40: EMM Manejo de los Abonos
    if (inst==0x40){
      lastInst=inst;

      
      // comprobar que utiliza MK
      if (p2&0x08){
        sw2=0x13;
        goto TX_Data;
      }

      // comprueba existencia de claves
      tmp = keyExist(p1, p2&0x0f);
      
      if (tmp==0x00){
        // No existe clave primaria 90 1D
        sw2=0x1D;
        goto TX_Data;
      } else
      if (tmp==0x50 && (p1&0x10)){
        // No existe clave secundaria 90 1F
        sw2=0x1F;
        goto TX_Data;
      }

      // ignoramos a SECA ( C1 40 00 81 17 )
      if (p1&0x0f){
        getKey(p1,p2);
        decode(p2,p3);
        
        if (signatureCheck(p1,p3)){
          processNanos(p1,kEMM);
        } else {sw2=02;}
      }
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x1A: Lectura de los Indices de claves
    if (inst==0x1A){
      u08 k;

      lastInst=inst;


      // FF FF 0B 40 00
      ee_readString(buffer, EE_AVAIL_KEYS, 5);

      for (p2=0, tmp=5; p2<16; p2++){
        k=keyExist(p1,p2)|p2;
        if (k&0xf0){
          // Nota: tiene en cuenta espacio en SECA  
          //if(p1 | (p2 < 8)){ 
            buffer[tmp++]=k;
          //}
        }
      }
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x5A: Lectura de claves encriptadas
    if (inst==0x5A){
      lastInst=inst;

      // comprueba existencia de claves
      // Nota: tiene en cuenta espacio en SECA
      //tmp = (p1 | (p2 < 8))? keyExist(p1,p2&0x0f) : 0x00;
      
      tmp = keyExist(p1,p2&0x0f);
      
      if (tmp==0x00){
        // No existe clave primaria 90 1D
        sw2=0x1d;
      } else
      if (tmp==0x50 && (p1&0x10)){
        // No existe clave secundaria 90 1F
        sw2=0x1F;
      } else {
        getKey(p1,p2);

        // Preparar 8*0x00
        fillBuffer(0,16,0x00);
        encript(Key,buffer);
      } 
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x0A: Sistema de Funcionamiento de la Tarjeta
    if (inst==0x0A){
      lastInst=inst;
      
      _memcpy(buffer,version,TOTAL_VERSION);
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x22: Lectura de eeprom
    // C1 22 P1 P2 P3
    // P1 Low Addr
    // P2 High Addr
    // P3 Bytes to read
    if ( (inst==0x22) && support_phoenix){
      ee_readString(buffer,(((u16)p1)<<8)+p2,p3);
    } else
    // - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    // INSTRUCCION 0x20: Escritura en eeprom
    // C1 20 P1 P2 P3
    // P1 Low Addr
    // P2 High Addr
    // P3 Bytes to write
    if ( (inst==0x20) && support_phoenix){
      RED_INFO_LED_ON;
      ee_writeString((((u16)p1)<<8)+p2,buffer,p3);
      RED_INFO_LED_OFF;
    } else {
    // UNKNOW INSTRUCTION
      lastInst=0;
      sw1=0x6D;
    }

    TX_Data:

    #ifdef MODO_LOG
    if (modo_log && !support_phoenix){ 
      capturar_inst(cla,inst,p1,p2,p3);
    }
    #endif
    
    if(inst&2){
      // CARD->CAM
      sendString(buffer,p3);
    } 

    (*ptrSend)(sw1);
    (*ptrSend)(sw2);

    #ifdef MODO_LOG
    if (ins_capturadas){
        ORANGE_INFO_LED_ON;
    }
    #endif


 } // end of while(1){}


  return 0;
} // end of main()
